From: Yang Yingliang <yangyingliang@huawei.com>
To: <linux-arm-kernel@lists.infradead.org>, <linux-kernel@vger.kernel.org>
Cc: <catalin.marinas@arm.com>, <will@kernel.org>,
<guohanjun@huawei.com>, <yangyingliang@huawei.com>
Subject: [PATCH 3/3] arm64: lib: improve copy performance when size is less than 128 and ge 64 bytes
Date: Tue, 23 Mar 2021 15:34:32 +0800 [thread overview]
Message-ID: <20210323073432.3422227-4-yangyingliang@huawei.com> (raw)
In-Reply-To: <20210323073432.3422227-1-yangyingliang@huawei.com>
When copy less than 128 and ge than 64 bytes, add src/dst after
load and store 64 bytes to improve performance.
Copy 127 bytes cost on Kunpeng920 (ms):
Without this patch:
memcpy: 14.62 copy_from_user: 14.23 copy_to_user: 14.42
With this patch:
memcpy: 13.85 copy_from_user: 13.26 copy_to_user: 13.84
It's about 5.27% improvement in memcpy().
Signed-off-by: Yang Yingliang <yangyingliang@huawei.com>
---
arch/arm64/lib/copy_template.S | 18 ++++++++++--------
1 file changed, 10 insertions(+), 8 deletions(-)
diff --git a/arch/arm64/lib/copy_template.S b/arch/arm64/lib/copy_template.S
index c3cd6f84c9c0..a9cbd47473f0 100644
--- a/arch/arm64/lib/copy_template.S
+++ b/arch/arm64/lib/copy_template.S
@@ -132,14 +132,16 @@ D_h .req x14
* Less than 128 bytes to copy, so handle 64 here and then jump
* to the tail.
*/
- ldp1 A_l, A_h, src, #16
- stp1 A_l, A_h, dst, #16
- ldp1 B_l, B_h, src, #16
- ldp1 C_l, C_h, src, #16
- stp1 B_l, B_h, dst, #16
- stp1 C_l, C_h, dst, #16
- ldp1 D_l, D_h, src, #16
- stp1 D_l, D_h, dst, #16
+ ldp2 A_l, A_h, src, #0, #8
+ stp2 A_l, A_h, dst, #0, #8
+ ldp2 B_l, B_h, src, #16, #24
+ ldp2 C_l, C_h, src, #32, #40
+ stp2 B_l, B_h, dst, #16, #24
+ stp2 C_l, C_h, dst, #32, #40
+ ldp2 D_l, D_h, src, #48, #56
+ stp2 D_l, D_h, dst, #48, #56
+ add src, src, #64
+ add dst, dst, #64
tst count, #0x3f
b.ne .Ltail63
--
2.25.1
prev parent reply other threads:[~2021-03-23 7:32 UTC|newest]
Thread overview: 10+ messages / expand[flat|nested] mbox.gz Atom feed top
2021-03-23 7:34 [PATCH 0/3] arm64: lib: improve copy performance Yang Yingliang
2021-03-23 7:34 ` [PATCH 1/3] arm64: lib: introduce ldp2/stp2 macro Yang Yingliang
2021-03-23 7:34 ` [PATCH 2/3] arm64: lib: improve copy performance when size is ge 128 bytes Yang Yingliang
2021-03-23 12:08 ` Robin Murphy
2021-03-23 13:32 ` Will Deacon
2021-03-23 14:28 ` Robin Murphy
2021-03-23 15:03 ` Catalin Marinas
2021-03-24 16:38 ` David Laight
2021-03-24 19:36 ` Robin Murphy
2021-03-23 7:34 ` Yang Yingliang [this message]
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20210323073432.3422227-4-yangyingliang@huawei.com \
--to=yangyingliang@huawei.com \
--cc=catalin.marinas@arm.com \
--cc=guohanjun@huawei.com \
--cc=linux-arm-kernel@lists.infradead.org \
--cc=linux-kernel@vger.kernel.org \
--cc=will@kernel.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
all inboxes | Powered by JetHome®